package dinamiche;

import java.util.*;

public class OrderedList<T>
implements List<T>
{
	//protected LinkedList<T> l;
	private List<T> l;

	public OrderedList()
	{
		l = new ArrayList<T>();
		//l = new LinkedList<T>();
	}

	public int size()
	{
		return l.size();
	}

	public boolean isEmpty()
	{
		return l.isEmpty();
	}

	public boolean contains(Object o)
	{
		return l.contains(o);
	}

	public boolean add(T e)
	{
		throw new UnsupportedOperationException("Supporta solo inserimento in ordine");
	}

	public boolean addInOrder(T e)
	{
		if(! (e instanceof Comparable))
		{
			throw new IllegalArgumentException("il parametro non e' di tipo Comparable");
		}
		Comparable<T> ec = (Comparable<T>)e;
		ListIterator<T> li = l.listIterator();
		while(li.hasNext())
		{
			T tmp = li.next();
			if(ec.compareTo(tmp) < 0)
			{
				li.previous();
				li.add(e);
				return true;
			}
		}
		li.add(e);
		return true;
	}

	public boolean remove(Object o)
	{
		return l.remove(o);
	}

	public boolean containsAll(Collection<?> c)
	{
		return l.containsAll(c);
	}

	public boolean addAll(Collection<? extends T> c)
	{
		throw new UnsupportedOperationException("Inserimento bulk non in ordine non supportato");
	}

	public boolean addAllInOrder(Collection<? extends T> c)
	{
		Iterator<? extends T> iter = c.iterator();
		while(iter.hasNext())
		{
			this.addInOrder(iter.next());
		}
		return true;
	}

	public boolean addAll(int index, Collection<? extends T> c)
	{
		throw new UnsupportedOperationException("Inserimento bulk non in ordine non supportato");
	}

	public boolean removeAll(Collection<?> c)
	{
		return l.removeAll(c);
	}

 public boolean retainAll(Collection<?> c)
 {
	boolean retval = false;
 	Iterator<T> iter = l.iterator();
	while(iter.hasNext())
	{
		T tmp = iter.next();
		if( !(c.contains(tmp)))
		{
			iter.remove();
			retval = true;
		}
	}
	return retval;

	/*
	return l.retainAll(c);
	*/
 }

 public void clear()
 {
 	l.clear();
 }

 public int hashCode()
 {
 	return toString().hashCode();
 }

 public String toString()
 {
 	return l.toString();
 }

 public boolean equals(Object o)
 {
 	if(! (o instanceof OrderedList))
	{
		return false;
	}
 	return l.equals( ((OrderedList)o).l);
 }

 public T get(int index)
 {
 	return l.get(index);
 }

 public T set(int index, T element)
	{
		throw new UnsupportedOperationException("Sostituzione in posizione non in ordine non supportato");
	}

	public void add(int index, T element)
	{
		throw new UnsupportedOperationException("Inserimento in posizione non in ordine non supportato");
	}

	public T remove(int index)
	{
		return l.remove(index);
	}

	public Object[] toArray()
	{
		return l.toArray();
	}

	public <T> T[] toArray(T[] a)
	{
		return l.toArray(a);
	}

    /**
     * Returns a list iterator of the elements in this list (in proper
     * sequence), starting at the specified position in this list.  The
     * specified index indicates the first element that would be returned by
     * an initial call to the <tt>next</tt> method.  An initial call to
     * the <tt>previous</tt> method would return the element with the
     * specified index minus one.
     *
     * @param index index of first element to be returned from the
     *		    list iterator (by a call to the <tt>next</tt> method).
     * @return a list iterator of the elements in this list (in proper
     * 	       sequence), starting at the specified position in this list.
     * @throws IndexOutOfBoundsException if the index is out of range (index
     *         &lt; 0 || index &gt; size()).
     */
    public ListIterator<T> listIterator(int index)
		{
			return new OrderedIterator<T>(l.listIterator(index));
		}

    /**
     * Returns a list iterator of the elements in this list (in proper
     * sequence).
     *
     * @return a list iterator of the elements in this list (in proper
     * 	       sequence).
     */
    public ListIterator<T> listIterator()
		{
			return listIterator(0);
		}

    /**
     * Returns an iterator over the elements in this list in proper sequence.
     *
     * @return an iterator over the elements in this list in proper sequence.
     */
    public Iterator<T> iterator()
		{
			return listIterator();
		}


    // Positional Access Operations

    // Search Operations

    /**
     * Returns the index in this list of the first occurrence of the specified
     * element, or -1 if this list does not contain this element.
     * More formally, returns the lowest index <tt>i</tt> such that
     * <tt>(o==null ? get(i)==null : o.equals(get(i)))</tt>,
     * or -1 if there is no such index.
     *
     * @param o element to search for.
     * @return the index in this list of the first occurrence of the specified
     * 	       element, or -1 if this list does not contain this element.
     * @throws ClassCastException if the type of the specified element
     * 	       is incompatible with this list (optional).
     * @throws NullPointerException if the specified element is null and this
     *         list does not support null elements (optional).
     */
    public int indexOf(Object o)
		{
			return l.indexOf(o);
		}

    /**
     * Returns the index in this list of the last occurrence of the specified
     * element, or -1 if this list does not contain this element.
     * More formally, returns the highest index <tt>i</tt> such that
     * <tt>(o==null ? get(i)==null : o.equals(get(i)))</tt>,
     * or -1 if there is no such index.
     *
     * @param o element to search for.
     * @return the index in this list of the last occurrence of the specified
     * 	       element, or -1 if this list does not contain this element.
     * @throws ClassCastException if the type of the specified element
     * 	       is incompatible with this list (optional).
     * @throws NullPointerException if the specified element is null and this
     *         list does not support null elements (optional).
     */
    public int lastIndexOf(Object o)
		{
			return l.lastIndexOf(o);
		}

    // View

    /**
     * Returns a view of the portion of this list between the specified
     * <tt>fromIndex</tt>, inclusive, and <tt>toIndex</tt>, exclusive.  (If
     * <tt>fromIndex</tt> and <tt>toIndex</tt> are equal, the returned list is
     * empty.)  The returned list is backed by this list, so non-structural
     * changes in the returned list are reflected in this list, and vice-versa.
     * The returned list supports all of the optional list operations supported
     * by this list.<p>
     *
     * This method eliminates the need for explicit range operations (of
     * the sort that commonly exist for arrays).   Any operation that expects
     * a list can be used as a range operation by passing a subList view
     * instead of a whole list.  For example, the following idiom
     * removes a range of elements from a list:
     * <pre>
     *	    list.subList(from, to).clear();
     * </pre>
     * Similar idioms may be constructed for <tt>indexOf</tt> and
     * <tt>lastIndexOf</tt>, and all of the algorithms in the
     * <tt>Collections</tt> class can be applied to a subList.<p>
     *
     * The semantics of the list returned by this method become undefined if
     * the backing list (i.e., this list) is <i>structurally modified</i> in
     * any way other than via the returned list.  (Structural modifications are
     * those that change the size of this list, or otherwise perturb it in such
     * a fashion that iterations in progress may yield incorrect results.)
     *
     * @param fromIndex low endpoint (inclusive) of the subList.
     * @param toIndex high endpoint (exclusive) of the subList.
     * @return a view of the specified range within this list.
     * 
     * @throws IndexOutOfBoundsException for an illegal endpoint index value
     *     (fromIndex &lt; 0 || toIndex &gt; size || fromIndex &gt; toIndex).
     */
    public List<T> subList(int fromIndex, int toIndex)
		{
			return l.subList(fromIndex, toIndex);
		}


		public static void main(String[] argv)
		{
			OrderedList<String> l = new OrderedList<String>();
			int i = 0;
			for(i=0;i<argv.length;i++)
			{
				l.addInOrder(argv[i]);
			}

			System.out.println(l);
			ListIterator iter = l.listIterator();

			while(iter.hasNext())
			{
				System.out.println(iter.next());
				iter.remove();
				System.out.println(l);
			}

		}

}
